/*======================================================================================================================
 *      File: calibrate.c
 *   Project: Parts Scale
 * Processor: ATtiny85
 * Frequency: 16.5 Mhz
 *    Author: eX Systems
 *     Email: jaredjulien@exsystems.net
 * Copyright: (c) 2018 eX Systems
 *
 * --------------------------------------------------------------------------------------------------------------------
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
 * OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * --------------------------------------------------------------------------------------------------------------------
 *
 * Date        Rev   Author    Change Description
 * ==========  ====  ========  ========================================================================================
 * 2018-02-11  1     Jared     Initial file creation based upon USB ignition switch project
 * --------------------------------------------------------------------------------------------------------------------
 */

#include <avr/io.h>
#include <avr/interrupt.h>

#include "usbdrv/usbdrv.h"
#include "macros.h"


//======================================================================================================================
// USB Callback Functions (called from usbdrv)
//----------------------------------------------------------------------------------------------------------------------
void usbEventResetReady(void)
{
	uint16_t frameLength;
	uint16_t targetLength = (unsigned)(1499 * (double)F_CPU / 10.5e6 + 0.5);
	uint16_t bestDeviation = 9999;
	uint8_t trialCal;
	uint8_t bestCal = 0;
	uint8_t step;
	uint8_t region;

	/* Disable interrupts during oscillator calibration since
	 * usbMeasureFrameLength() counts CPU cycles.
	 */
	cli();

	// do a binary search in regions 0-127 and 128-255 to get optimum OSCCAL
	for (region = 0U; region <= 1U; region++)
	{
		frameLength = 0U;
		trialCal = (region == 0U) ? 0U : 128U;

		for (step = 64U; step > 0U; step >>= 1U)
		{
			if (frameLength < targetLength) // true for initial iteration
			{
				trialCal += step; // frequency too low
			}
			else
			{
				trialCal -= step; // frequency too high
			}

			OSCCAL = trialCal;
			frameLength = usbMeasureFrameLength();

			if (abs(frameLength - targetLength) < bestDeviation)
			{
				bestCal = trialCal; // new optimum found
				bestDeviation = abs(frameLength -targetLength);
			}
		}
	}

	OSCCAL = bestCal;

	sei();

	// eeprom_write_byte(0, OSCCAL);   /* store the calibrated value in EEPROM */
}

/* End of File */
