
//======================================================================================================================
// Includes
//----------------------------------------------------------------------------------------------------------------------
#include "ds18b20.h"
#include "ds1wire.h"
#include "pins.h"


//======================================================================================================================
// Constants
//----------------------------------------------------------------------------------------------------------------------
#define DS18B20_CMD_SEARCHROM   0xF0
#define DS18B20_CMD_SKIPROM     0xCC
#define DS18B20_CMD_MATCHROM    0x55
#define DS18B20_CMD_READROM     0x33
#define DS18B20_CMD_CONVERTT    0x44
#define DS18B20_CMD_READSCRPAD  0xBE
#define DS18B20_CMD_READPOWER   0xB4


//======================================================================================================================
// Private Functions
//----------------------------------------------------------------------------------------------------------------------
static void ds18b20write(uint8_t val)
{
	uint8_t idx;
	for (idx = 0; idx < 8; idx++)
	{
		if (val & 0x01)
		{
			ds1wire_send1(B, PIN_TEMP);
		}
		else
		{
			ds1wire_send0(B, PIN_TEMP);
		}
		val >>= 1;
	}
}

//----------------------------------------------------------------------------------------------------------------------
static uint8_t ds18b20read(void)
{
	uint8_t idx;
	uint8_t res = 0;
	for (idx = 0; idx < 8; idx++)
	{
		res |= ((ds1wire_read(B, PIN_TEMP)) << idx);
	}

	return res;
}


//======================================================================================================================
// Public Functions
//----------------------------------------------------------------------------------------------------------------------
void ds18b20_init(void)
{
	ds1wire_init(B, PIN_TEMP);
	ds1wire_reset(B, PIN_TEMP);
}

//----------------------------------------------------------------------------------------------------------------------
void ds18b20_startConversion(void)
{
	// Reset the bus.
	ds1wire_reset(B, PIN_TEMP);

	// Skip ROM.
	ds18b20write(DS18B20_CMD_SKIPROM);

	// Issue 'start temperature conversion'.
	ds18b20write(DS18B20_CMD_CONVERTT);
}

//----------------------------------------------------------------------------------------------------------------------
uint8_t ds18b20_getTemp(int16_t * output)
{
	uint8_t idx;
	uint8_t crc = 0;
	uint8_t value;
	uint16_t temp = 0;

	// Ensure that the conversion is complete.
	if (ds1wire_read(B, PIN_TEMP) == 0)
	{
		// If device returns all zeros then the conversion is still pending.
		return DS18B20_ERR_PENDING;
	}

	ds1wire_reset(B, PIN_TEMP);

	// Skip ROM.
	ds18b20write(DS18B20_CMD_SKIPROM);

	// Read Scratchpad memory.
	ds18b20write(DS18B20_CMD_READSCRPAD);
	crc = 0;
	for (idx = 0; idx < 9; idx++)
	{
		value = ds18b20read();
		if (idx == 0)
		{
			temp = (int16_t)((uint16_t)value) << 3;
		}
		else if (idx == 1)
		{
			temp |= (int16_t)((uint16_t)value) << 11;
		}
		crc = ds1wire_calccrc8(crc, value);
	}

	if (crc == 0)
	{
		*output = temp;
		return DS18B20_ERR_OK;
	}
	else
	{
		return DS18B20_ERR_INVALID_CRC;
	}
}

/* End of File */
