#ifndef HX711_h
#define HX711_h

#include <avr/io.h>

// Allows to set the pins and gain later than in the constructor
void hx711_begin(uint8_t gain);

// check if HX711 is ready
// from the datasheet: When output data is not ready for retrieval, digital output pin DOUT is high. Serial clock
// input PD_SCK should be low. When DOUT goes to low, it indicates data is ready for retrieval.
uint8_t hx711_is_ready(void);

// set the gain factor; takes effect only after a call to read()
// channel A can be set for a 128 or 64 gain; channel B has a fixed 32 gain
// depending on the parameter, the channel is also set to either A or B
void hx711_set_gain(uint8_t gain);

// waits for the chip to be ready and returns a reading
uint32_t hx711_read(void);

// puts the chip into power down mode
void hx711_power_down(void);

// wakes up the chip after power down mode
void hx711_power_up(void);

#endif /* HX711_h */